# vim:set et sts=4 sw=4:
#
# ibus - The Input Bus
#
# Copyright (c) 2010 Takao Fujiwara <takao.fujiwara1@gmail.com>
# Copyright (c) 2007-2010 Peng Huang <shawn.p.huang@gmail.com>
# Copyright (c) 2007-2010 Red Hat, Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA  02111-1307  USA

__all__ = (
        "XKBLayout",
    )

import os

XKB_COMMAND = "/usr/libexec/ibus-xkb"

class XKBLayout():
    def __init__(self, config = None, command=XKB_COMMAND):
        self.__config = config
        self.__command = command
        self.__use_xkb = True
        if self.__command == None:
            self.__use_xkb = False
        self.__default_layout = self.get_layout()
        self.__default_model = self.get_model()
        self.__default_option = self.get_option()
        self.__xkb_latin_layouts = []
        if config != None:
            self.__xkb_latin_layouts = list(self.__config.get_value("general",
                                                                    "xkb_latin_layouts",
                                                                    []))


    def __get_model_from_layout(self, layout):
        layout_array = layout.split(',')
        option_array = []
        is_bracket = False
        for i, l in enumerate(layout_array):
            option_array.append("")
            left_bracket = l.find('(')
            right_bracket = l.find(')')
            if left_bracket >= 0 and right_bracket > left_bracket:
                is_bracket = True
                layout_array[i] = l[:left_bracket]
                option_array[i] = l[left_bracket + 1:right_bracket]
        if is_bracket == False:
            return (layout, "default")
        layout = ','.join(layout_array)
        option = ','.join(option_array)
        return (layout, option)

    def use_xkb(self, enable):
        if self.__command == None:
            return
        self.__use_xkb = enable

    def get_layout(self):
        if not self.__use_xkb:
            return None
        exec_command = "%s %s" % (self.__command, "--get")
        retval = None
        for line in os.popen(exec_command).readlines():
            line = line.strip()
            if line.startswith("layout: "):
                retval = line[len("layout: "):]
                break
        return retval

    def get_model(self):
        if not self.__use_xkb:
            return None
        exec_command = "%s %s" % (self.__command, "--get")
        retval = None
        for line in os.popen(exec_command).readlines():
            line = line.strip()
            if line.startswith("model: "):
                retval = line[len("model: "):]
                break
        return retval

    def get_option(self):
        if not self.__use_xkb:
            return None
        exec_command = "%s %s" % (self.__command, "--get")
        retval = None
        for line in os.popen(exec_command).readlines():
            line = line.strip()
            if line.startswith("option: "):
                retval = line[len("option: "):]
                break
        return retval

    def set_layout(self, layout="default", model="default", option="default"):
        if not self.__use_xkb:
            return
        if layout == None:
            return
        layout = str(layout)
        if layout == "default":
            layout = self.__default_layout
        if model != None:
            model = str(model)
            if model == "default":
                (layout, model) = self.__get_model_from_layout(layout)
            if model == "default":
                model = self.__default_model
        if option != None:
            option = str(option)
            if option == "default":
                option = self.__default_option
        need_us_layout = False
        for latin_layout in self.__xkb_latin_layouts:
            latin_layout = str(latin_layout)
            if layout == latin_layout:
                need_us_layout = True
                break
            if model != None and layout + '(' + model + ')' == latin_layout:
                need_us_layout = True
                break
        if need_us_layout:
            layout = layout + ",us"
            if model != None:
                model = model + ","
        if layout == self.get_layout() and \
           model == self.get_model() and \
           option == self.get_option():
            return
        args = []
        args.append(self.__command)
        args.append(os.path.basename(self.__command))
        args.append("--layout")
        args.append(layout)
        if model != None:
            args.append("--model")
            args.append(model)
        if option != None:
            args.append("--option")
            args.append(option)
        pid = os.spawnl(os.P_NOWAIT, *args)
        os.waitpid(pid, 0)

    def set_default_layout(self, layout="default", model="default"):
        if not self.__use_xkb:
            return
        if layout == 'default':
            self.__default_layout = self.get_layout()
            self.__default_model = self.get_model()
        else:
            if model == 'default':
                (layout, model) = self.__get_model_from_layout(layout)
            self.__default_layout = layout
            if model == 'default':
                self.__default_model = None
            else:
                self.__default_model = model

    def set_default_option(self, option="default"):
        if not self.__use_xkb:
            return
        if option == 'default':
            self.__default_option = self.get_option()
        else:
            self.__default_option = option

    def reload_default_layout(self):
        if not self.__use_xkb:
            return
        self.__default_layout = self.get_layout()
        self.__default_model = self.get_model()
        self.__default_option = self.get_option()
